// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#ifndef AT_COLOR_H
#define AT_COLOR_H

class ATColor
{
	private:
		float _r;
		float _g;
		float _b;
		float _a;
		
		static ATColor _BACK_GRND;
		static ATColor _ACTIVE_BACK_GRND;
		static ATColor _ELEMENT_FRAME;
		static ATColor _ACTIVE_FRAME;
		
		static ATColor _DARK;
		static ATColor _LIGHT;
		
	public:
			ATColor() : 
				_r(0), _g(0), _b(0), _a(0) {}
		
			ATColor( float r, float g, float b, float a ) :
				_r(r), _g(g), _b(b), _a(a) {}
		
			ATColor( float r, float g, float b ) :
				_r(r), _g(g), _b(b), _a(1) {}
		
			ATColor( const ATColor & c ) :
				_r(c._r), _g(c._g), _b(c._g), _a(c._a) {}
				
			static const ATColor & BackGrnd() { return _BACK_GRND; }
			static const ATColor & ActiveBackGrnd() { return _ACTIVE_BACK_GRND; }
			static const ATColor & ElementFrame() { return _ELEMENT_FRAME; }
		
			static const ATColor & ElementDark() { return _DARK; }
			static const ATColor & ElementLight() { return _LIGHT; }
			static const ATColor & ActiveElement() { return _ACTIVE_FRAME; }
		
			ATColor GetGradientIncrement( const ATColor & to, int steps ) const;
		
			void IncremenColor( const ATColor & incr );
			
	public:
			float Red() const { return _r; }
			float Green() const { return _g; }
			float Blue() const { return _b; }
			float Alpha() const { return _a; }
};


#endif //!AT_COLOR_H
