// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#import "Apple Tree_Prefix.pch"
#import "ATController.h"
#import "ThreadWorker.h"

//-----------------------------------------------------------------------------
// private methods declaration
//-----------------------------------------------------------------------------

@interface ATController (Private)
-(BOOL) isValidDirectory: (NSString*) path;
-(void) startProgressSheet;
-(void) endProgressSheet;
-(NSString*) getUserHomeDir;

-(id) startAnalyzing:(id)userInfo workerThread:(ThreadWorker *)tw;
-(id) finishedAnalyzing: (id) param;

// Methods for Open Directory sheet
// handler for sheet ending
-(void) showBrowseSheet;

-(void) openPanelDidEnd:(NSOpenPanel *)panel
				returnCode:(int)returnCode
			  contextInfo:(void *)contextInfo;

@end

//-----------------------------------------------------------------------------
// public methods implemenation
//-----------------------------------------------------------------------------


@implementation ATController

//-----------------------------------------------------------------------------
- (id) init
//-----------------------------------------------------------------------------
{
	[super init];
	didPathInit = NO;
	return self;
}

//-----------------------------------------------------------------------------
- (void) awakeFromNib
//-----------------------------------------------------------------------------
{
	if ( !didPathInit )
	{
		didPathInit = YES;
		[targetDirectory setStringValue:[self getUserHomeDir]];
	}
	
	if ( progressBar )
	{
		[progressBar startAnimation:nil];
	}
	
	[treeView setController:self];
}

//-----------------------------------------------------------------------------
- (IBAction)analyze:(id)sender
//-----------------------------------------------------------------------------
{
	NSString * path = [targetDirectory stringValue];
	if ( YES==[self isValidDirectory:path] )
	{
		[self startProgressSheet];
		
		[ThreadWorker workOn:self
				  withSelector:@selector(startAnalyzing:workerThread:)
					 withObject:path
				didEndSelector:@selector(finishedAnalyzing:)];
	}
}

//-----------------------------------------------------------------------------
- (IBAction)browse:(id)sender
//-----------------------------------------------------------------------------
{
	[self showBrowseSheet];
}

//-----------------------------------------------------------------------------
- (IBAction)cancelAnalyze: (id)sender
//-----------------------------------------------------------------------------
{
	[analyzer CancelOperation];
	[NSApp endSheet:progressPanel];
}

//-----------------------------------------------------------------------------
- (NSTextField*) statusText
//-----------------------------------------------------------------------------
{
	return statusText;
}

//-----------------------------------------------------------------------------
- (void) setStatusText:(NSTextField*)text
//-----------------------------------------------------------------------------
{
	statusText = text;
}

@end

//-----------------------------------------------------------------------------
// private methods implemenation
//-----------------------------------------------------------------------------


@implementation ATController (Private)
//-----------------------------------------------------------------------------
-(BOOL) isValidDirectory: (NSString*) path
//-----------------------------------------------------------------------------
{
	BOOL isValid = NO;
	
	if ( path != nil )
	{
		NSFileManager *fileManager = [NSFileManager defaultManager];
		if ( [fileManager fileExistsAtPath:path] )
		{
			isValid = YES;
		}
	}
	
	return isValid;
}

//-----------------------------------------------------------------------------
-(void) startProgressSheet
//-----------------------------------------------------------------------------
{
	if ( !progressPanel )
		[NSBundle loadNibNamed: @"ProgressSheet" owner: self];
	
	[progressBar setUsesThreadedAnimation:NO];
	
	[NSApp beginSheet: progressPanel
		modalForWindow: appWindow
		 modalDelegate: self
		didEndSelector: @selector(sheetDidEnd:returnCode:contextInfo:)
			contextInfo: nil];
}

//-----------------------------------------------------------------------------
-(void) endProgressSheet
//-----------------------------------------------------------------------------
{
	[NSApp endSheet:progressPanel];
}

//-----------------------------------------------------------------------------
- (void)sheetDidEnd:(NSWindow *)sheet 
			returnCode:(int)returnCode 
		  contextInfo:(void *)contextInfo
//-----------------------------------------------------------------------------
{
	[sheet orderOut:self];
}

//-----------------------------------------------------------------------------
-(NSString*) getUserHomeDir
//-----------------------------------------------------------------------------
{
	NSString * homeDir = [NSString stringWithString:@"~"];
	return [homeDir stringByExpandingTildeInPath];
}

//-----------------------------------------------------------------------------
- (void) showBrowseSheet 
//-----------------------------------------------------------------------------
{
	
	NSOpenPanel * openPanel = [[NSOpenPanel openPanel] retain];
	[openPanel setCanChooseDirectories:YES];
	[openPanel setCanChooseFiles:NO];
	
	[openPanel beginSheetForDirectory:nil
										  file:nil
										 types:nil
							 modalForWindow:appWindow
							  modalDelegate:self
							 didEndSelector:@selector(openPanelDidEnd:
															  returnCode:
															  contextInfo:)
						  contextInfo:NULL];
}

//-----------------------------------------------------------------------------
-(void)openPanelDidEnd:(NSOpenPanel *)panel
				returnCode:(int)returnCode
			  contextInfo:(void *)contextInfo
//-----------------------------------------------------------------------------
{
	if (returnCode == NSOKButton)
	{ 
		NSArray * resultFiles = [panel filenames];
		if ( [resultFiles count] > 0 )
		{
			NSString * path = [resultFiles objectAtIndex:0];
			[targetDirectory setStringValue:path];
		}
	}
	[panel release];
}

//-----------------------------------------------------------------------------
-(id)startAnalyzing:(id)userInfo workerThread:(ThreadWorker *)tw
//-----------------------------------------------------------------------------
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSString * path = userInfo;
	[analyzer analyze:path];
	[pool release];
	return nil;
}

//-----------------------------------------------------------------------------
-(id) finishedAnalyzing: (id) param
//-----------------------------------------------------------------------------
{
	[self endProgressSheet];
	[appWindow display];
	return nil;
}
@end
