// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#include "ATCoordinateMap.h"

//-----------------------------------------------------------------------------
ATCoordinateMap::ATCoordinateMap()
//-----------------------------------------------------------------------------
{
}


//-----------------------------------------------------------------------------
void ATCoordinateMap::AddCoordinateRect( /*[in]*/ const CGRect & rect, 
													  /*[in]*/ ElementMap::iterator & element,
													  /*[in]*/  const ATColor & color)
//-----------------------------------------------------------------------------
{
	CoordContainer::iterator pos = _drawElements.insert( CoordContainer::value_type(element->second.Id(), ATDrawElement(rect, element, color)) ).first;
	if ( !(element->second.IsDir()) )
	{
		_xCoords.insert( CoordMap::value_type(rect.origin.x, pos) );
	}
}

//-----------------------------------------------------------------------------
void ATCoordinateMap::clear()
//-----------------------------------------------------------------------------
{
	_drawElements.clear();
	_xCoords.clear();
}

//-----------------------------------------------------------------------------
CoordContainer::iterator ATCoordinateMap::FindElement( const NSPoint & p )
//-----------------------------------------------------------------------------
{
	CGPoint point = CGPointMake( p.x, p.y );
	return FindElement( point );
}

//-----------------------------------------------------------------------------
CoordContainer::iterator ATCoordinateMap::FindElement( const CGPoint & p )
//-----------------------------------------------------------------------------
{
	CoordContainer::iterator pos = end();
	
	CoordMap::iterator xPos = _xCoords.lower_bound( p.x );
	if ( xPos!=_xCoords.end() )
	{
		while ( !IsInRect(p, xPos->second->second.Rect())
				  && (xPos!=_xCoords.begin()) )
		{
			--xPos;
		}
		pos = xPos->second;
	}
	
	return pos;
}

//-----------------------------------------------------------------------------
CoordContainer::iterator ATCoordinateMap::FindElement( const ATElementId & elementId )
//-----------------------------------------------------------------------------
{
	return _drawElements.find( elementId );
}

//-----------------------------------------------------------------------------
bool ATCoordinateMap::IsInRect( const CGPoint & p, const CGRect & rect )
//-----------------------------------------------------------------------------
{
	return CGRectContainsPoint(rect, p) ? true : false;
}

//-----------------------------------------------------------------------------
bool ATCoordinateMap::empty()
//-----------------------------------------------------------------------------
{
	return _drawElements.empty();
}