// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#ifndef ATDISK_ELEMENT_H
#define ATDISK_ELEMENT_H

#include <map>
#include <algorithm>
#include <string>
#include <memory>

#include "ATStringContainer.h"

class ATDiskElement;

typedef int ATElementId;

typedef std::multimap<float, ATDiskElement, std::greater<float> > ElementMap;
typedef std::auto_ptr<ATStringContainer> StringElemPtr;

class ATDiskElement
{
	private:
		static ATElementId _idSequence;
	
		int _id;
		float _bytes;
		NSString * _name;
		ElementMap _subElements;
		ATDiskElement * _pParent;
		StringElemPtr _pathName;
		
	public:
			ATDiskElement();
			ATDiskElement( const ATDiskElement & element );
			ATDiskElement( float bytes, NSString * name );
			~ATDiskElement();
			
			int Id() const { return _id; }
		
			void Name( NSString * name );
			inline NSString * Name () const { return _name; }
			
			void Bytes( float bytes );
			inline float Bytes() const { return _bytes; };
			
			void Parent( ATDiskElement * pParent );
			ATDiskElement * Parent() const { return _pParent; }
			
			void Clear();
			ATDiskElement & AddSubElement( NSString * name, float bytes );
			ATDiskElement & AddSubElement( ATDiskElement & element );
			
			inline ElementMap::iterator begin() { return _subElements.begin(); }
			inline ElementMap::iterator end()   { return _subElements.end();   }
			inline int size() const { return _subElements.size(); }
			
			inline bool IsDir() const { return !_subElements.empty(); }
			void WriteParentData();
			NSString * GetPathName();
			
			static void ResetIdSequence();

			
			std::string ToString();
			void WriteElementContent();
			
	private:
			ATElementId GetNextId();
				
			NSString * MakeSizeString();
			
};

#endif //!ATDISK_ELEMENT_H