// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#ifndef ATPAINTER_H
#define ATPAINTER_H

#include <Cocoa/Cocoa.h>
#include "Analyzer.h"
#include "ATDrawElement.h"
#include <vector>
#import "ATTreeView.h"
#include "ATColor.h"

typedef enum DrawDirection
{
	horizontal = 0x1, 
	vertical = 0x2
} DrawDirection;

class ATPainter
{
	public:
		ATPainter( CGRect rect, CGContextRef context, Analyzer & analyzer, ATTreeView * view );
		void DrawTree();
		
	protected:
		inline CGRect & MainRect()       { return _mainRect; }
		inline CGContextRef & Context()  { return _context;  }
		inline Analyzer & Data()         { return _analyzer; }
		
	private:
		void DrawNoData();
		void DrawXCenteredString( float y, NSString * text, NSString * font, NSColor * color,  int size );
		void DrawXCenteredStringAttribs( float y, NSString * text, NSMutableDictionary *attributes );
		
		void GetNextRowElements( ElementMap::iterator begin, ElementMap::iterator & pos, ElementMap::iterator end, const CGRect & rect, DrawDirection dir );
		
		inline void GetRowData( ElementMap::iterator start, ElementMap::iterator end, float & sum , float & minVal, float & maxVal );
		inline float GetWidth( CGRect & rect, DrawDirection dir );
		inline float GetRowChildRelation( ElementMap::iterator begin, ElementMap::iterator rowEnd, ElementMap::iterator & end );
		inline float GetRowChildRelation( ElementMap::iterator begin, ElementMap::iterator rowEnd, ElementMap::iterator & end, float & sum, float & rowSum );
		inline float GetRectRelation( ElementMap::iterator begin, ElementMap::iterator rowEnd, ElementMap::iterator & end, const CGRect & rect, DrawDirection dir );
		inline float DeltaRectRel( float rel );
		inline float GetRectRel( ElementMap::iterator pElement, float width, float height, float sum, DrawDirection dir );
		inline bool HasDrawSize( const CGRect & rect );
		
		void DrawData();
		void DrawElement( ATDiskElement & element, const CGRect & rect );
		void DrawRow( ElementMap::iterator begin, ElementMap::iterator rowEnd, ElementMap::iterator end, CGRect & rect, DrawDirection dir, CGRect & remainingRect, DrawElemVector & drawElements, ATColor & elementColor, const ATColor & incrColor );
		void DrawHorizontalRow( ElementMap::iterator begin, ElementMap::iterator & end, CGRect & rect, DrawElemVector & drawElements, ATColor & elementColor, const ATColor & incrColor );
		void DrawVerticalRow( ElementMap::iterator begin, ElementMap::iterator & end, CGRect & rect, DrawElemVector & drawElements, ATColor & elementColor, const ATColor & incrColor );
		void DrawRect( const CGRect & rect, const ATColor & color );
		void DrawFromCoordinates();
		void DrawBackGround( const CGRect & rect );
		void DrawParentRects( ATDiskElement & element, bool active );
		inline void DrawRectangle( const CGRect & rect, const ATColor & color );
		inline void DrawFilledRectangle( const CGRect & rect, const ATColor & color );
								  
		inline void CalculateDrawRects( CGRect & rect, float rel, DrawDirection dir, CGRect & drawRect, CGRect & remainingRect );
		inline float GetElementSum( ElementMap::iterator begin, ElementMap::iterator & end );
		
		inline DrawDirection GetDirection( const CGRect & rect );
		inline DrawDirection Inverse( DrawDirection dir );
		inline NSRect NSRectMake( const CGRect & rect );
		inline CGRect GetMainMinRect();
		
	private:
		CGRect _mainRect;
		CGContextRef _context;
		Analyzer & _analyzer;
		float _unit;
		ATTreeView * _view;
		bool _isLiveResize;
		CGPoint _cursorPoint;
};


#endif //!ATPAINTER_H