// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#import "Apple Tree_Prefix.pch"
#import "ATTreeView.h"
#import "ATPaintWrapper.h"
#import "ATController.h"


BOOL IsValidPoint( NSPoint * p )
{
	return ( p->x>0 && p->y>0 );
}

//-----------------------------------------------------------------------------
// private methods declaration
//-----------------------------------------------------------------------------

@interface ATTreeView (Private)

- (NSRect) growRect:(NSRect)rect g:(float) g;

@end

//-----------------------------------------------------------------------------
// public methods implementation
//-----------------------------------------------------------------------------

@implementation ATTreeView

//-----------------------------------------------------------------------------
- (id)initWithFrame:(NSRect)frameRect
//-----------------------------------------------------------------------------
{
	duringLiveResize = NO;
	
	if ((self = [super initWithFrame:frameRect]) != nil)
	{
		trackingTag = [self addTrackingRect:[self bounds] 
												owner:self 
											userData:nil
									  assumeInside:NO];

		lastPoint = NSMakePoint( 0, 0 );
		drawPoint = NSMakePoint( 0, 0 );
	}
	return self;
}

//-----------------------------------------------------------------------------
- (void)drawRect:(NSRect)rect
//-----------------------------------------------------------------------------
{
	[self removeTrackingRect:trackingTag];
	
	NSRect bounds = [self bounds];
	
	trackingTag = [self addTrackingRect:bounds
											owner:self 
										userData:nil
								  assumeInside:NO];
	
	CGContextRef myContext = [[NSGraphicsContext currentContext]graphicsPort];
	
	CGRect  cg = CGRectMake(NSMinX(bounds), NSMinY(bounds), 
								  NSWidth(bounds), NSHeight(bounds));
	
	ATPaintWrapper * painter = [ATPaintWrapper initWithContext:cg
																		context:myContext];
	[painter paintTree: [[self analyzer] wrapper] view:self];
}

//-----------------------------------------------------------------------------
- (ATAnalyzer*) analyzer
//-----------------------------------------------------------------------------
{
	return analyzer;
}

//-----------------------------------------------------------------------------
- (void)viewWillStartLiveResize
//-----------------------------------------------------------------------------
{
	[super viewWillStartLiveResize];
	duringLiveResize = YES;
}

//-----------------------------------------------------------------------------
- (void)viewDidEndLiveResize
//-----------------------------------------------------------------------------
{
	[super viewDidEndLiveResize];
	duringLiveResize = NO;
	[self setNeedsDisplay:YES];
}

//-----------------------------------------------------------------------------
- (BOOL) duringLiveResize
//-----------------------------------------------------------------------------
{
	return duringLiveResize;
}

//-----------------------------------------------------------------------------
- (void) setDuringLiveResize:(BOOL)liveResize
//-----------------------------------------------------------------------------
{
	duringLiveResize = liveResize;
}

/*
//-----------------------------------------------------------------------------
- (NSString *)view:(NSView *)view stringForToolTip:(NSToolTipTag)tag 
				 point:(NSPoint)point 
			 userData:(void *)userData
//-----------------------------------------------------------------------------
{
	NSString * result = nil;
	NSLog( @"stringForToolTip" );
	
	result = [analyzer GetElementString:point];
	
	return result;
}
*/

//-----------------------------------------------------------------------------
- (void)mouseMoved:(NSEvent *)theEvent
//-----------------------------------------------------------------------------
{
	[self removeAllToolTips];
	if ( [analyzer hasData] )
	{
		drawPoint = lastPoint;
		NSPoint p = [theEvent locationInWindow];
		lastPoint = [self convertPoint:p fromView:nil];
		
		[self setNeedsDisplayInRect:[analyzer getPointRect:drawPoint]];
		
		if ( IsValidPoint(&lastPoint) )
		{
			NSRect rect = [self bounds];
			if ( NSPointInRect(lastPoint, rect) )
			{
				[self setNeedsDisplayInRect:[analyzer getPointRect:lastPoint]];
				[self applyStatusText:[analyzer GetElementString:lastPoint]];
			}
		}
	}
}

//-----------------------------------------------------------------------------
- (void)mouseDragged:(NSEvent *)event
//-----------------------------------------------------------------------------
{
}

//-----------------------------------------------------------------------------
- (void)mouseEntered:(NSEvent *)theEvent
//-----------------------------------------------------------------------------
{
	[[self window] makeFirstResponder: self];
	[[self window] setAcceptsMouseMovedEvents: YES];
}

//-----------------------------------------------------------------------------
- (void)mouseExited:(NSEvent *)theEvent
//-----------------------------------------------------------------------------
{	
	[[self window] setAcceptsMouseMovedEvents: NO];
}

//-----------------------------------------------------------------------------
-(BOOL) acceptsFirstResponder
//-----------------------------------------------------------------------------
{
	return YES;
}

//-----------------------------------------------------------------------------
- (NSPoint) lastPoint
//-----------------------------------------------------------------------------
{
	return lastPoint;
}

//-----------------------------------------------------------------------------
- (NSPoint) drawPoint
//-----------------------------------------------------------------------------
{
	return drawPoint;
}

//-----------------------------------------------------------------------------
- (void) applyStatusText:(NSString*)statusText
//-----------------------------------------------------------------------------
{
	if ( controller!=nil )
	{
		ATController * ctrl = controller;
		if ( statusText )
			[[ctrl statusText] setStringValue:statusText];
	}
}

//-----------------------------------------------------------------------------
- (void) setController:(id)ctrl
//-----------------------------------------------------------------------------
{
	controller = ctrl;
}

//-----------------------------------------------------------------------------
- (BOOL)wantsDefaultClipping
//-----------------------------------------------------------------------------
{
	return NO;
}

//-----------------------------------------------------------------------------
- (BOOL)isOpaque
//-----------------------------------------------------------------------------
{
	return YES;
}

@end

//-----------------------------------------------------------------------------
// private methods implementation
//-----------------------------------------------------------------------------

@implementation ATTreeView (Private)

//-----------------------------------------------------------------------------
- (NSRect) growRect:(NSRect)rect g:(float)g
//-----------------------------------------------------------------------------
{
	NSRect result;
	result.origin.x = rect.origin.x > g ? rect.origin.x - g : 0;
	result.origin.y = rect.origin.y > g ? rect.origin.y - g  : 0;
	result.size.height = rect.size.height + g;
	result.size.width = rect.size.width + g;
	return result;
}


@end
