// 
// Copyright (C) 2005 Stefan Domnanovits
// mailto:stefan@domnanovits.at
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details

#ifndef ANALYZER_H
#define ANALYZER_H

#include <string>
#include <vector>

#include "ATDiskElement.h"
#include "ATStringDictionary.h"
#include "ATCoordinateMap.h"

typedef std::vector<ATDiskElement> ElementVector;

class Analyzer
{
	public:
		Analyzer();
		bool HasData() { return _hasData; }
	
		void Analyze( NSString * path );
		void CancelOperation();
		
		ATDiskElement & RootElement();
		
		void ClearCoordinates();
		bool HasCoordinates();
		ATCoordinateMap & Coordinates();
		CGRect GetPointRect( const CGPoint & p );
		CGRect GetPointRect( const NSPoint & p );
		
		bool GetPointElement( const NSPoint & p, CoordContainer::iterator & element );
		bool GetDrawElement( const ATElementId & elemenId, CoordContainer::iterator & drawElem );
		
	protected:
			void Clear();
			
			int AddSubElements( NSFileManager * fileManager, ATDiskElement & rootElement, NSString * path );
			bool IsDirectory( NSDictionary * attribs );
			bool IsLink( NSDictionary * attribs );
			float GetSize( NSDictionary * attribs );
			
			inline bool DoCancel() { return _doCancel; }
	
	private:
		bool _hasData;
		bool _doCancel;
		ATStringDictionary _stringDict;
		ATDiskElement _rootElement;
		ATCoordinateMap _coordinates;
};

#endif //!ANALYZER_H

